﻿using System;
using System.Collections.Generic;

using System.IO;
using System.Diagnostics;
using System.Threading;

namespace Mutation
{
  class Program
  {
    static Random ran = new Random(2);
    static void Main(string[] args)
    {
      try
      {
        Console.WriteLine("\nBegin super-simple mutation testing demo\n");

        string originalSourceFile = "..\\..\\Class1-Original.cs"; 
        string mutatedSourceFile = "..\\..\\..\\MathLib\\Class1.cs";
        string mutantProject = "..\\..\\..\\MathLib\\MathLib.csproj";
        string testProject = "..\\..\\..\\TestMutation\\TestMutation.csproj";
        string testExecutable = "..\\..\\..\\TestMutation\\bin\\Debug\\TestMutation.exe";
        string devenv = "C:\\Program Files (x86)\\Microsoft Visual Studio 9.0\\Common7\\IDE\\devenv.exe";

        Console.WriteLine("Determining possible mutation positions in MathLib.dll source code");
        List<int> positions = GetMutationPositions(originalSourceFile);
        int numberMutants = 2;

        for (int i = 0; i < numberMutants; ++i)
        {
          Console.WriteLine("\n=============");
          Console.WriteLine("Mutant # " + i);
          Console.WriteLine("\nSelecting random mutation position");
          int randomPosition = positions[ran.Next(0, positions.Count)];
          Console.WriteLine("Creating mutant source code file");
          CreateMutantSource(originalSourceFile, randomPosition, mutatedSourceFile);

          try
          {
            Console.WriteLine("\nBegin build mutant MathLib.dll");
            BuildMutant(mutantProject, devenv);
            Console.WriteLine("\nRebuilding test harness with mutated DLL");
            BuildTestProject(testProject, devenv);
            Console.WriteLine("\nTesting mutant DLL and writing results to log file");
            Console.WriteLine("Check log to make sure at least one test failure generated!");
            TestMutant(testExecutable);
          }
          catch
          {
            Console.WriteLine("Invalid mutant. Aborting.");
            continue;
          }
          Console.WriteLine("=============");
        }

        Console.WriteLine("\nMutation test run complete");
        //Console.ReadLine();
      }
      catch (Exception ex)
      {
        Console.WriteLine("Fatal: " + ex.Message);
        Console.ReadLine();
      }
    } // Main()

    static List<int> GetMutationPositions(string originalSourceFile)
    {
      StreamReader sr = File.OpenText(originalSourceFile);
      int ch = 0; int pos = 0;
      List<int> list = new List<int>();
      while ((ch = sr.Read()) != -1) {
        if ((char)ch == '>' || (char)ch == '<')
          list.Add(pos);
        ++pos;
      }
      sr.Close();
      return list;
    } // GetMutationPositions

    static void CreateMutantSource(string originalSourceFile, int mutatePosition, string mutatedSourceFile)
    {
      FileStream ifs = new FileStream(originalSourceFile, FileMode.Open);
      StreamReader sr = new StreamReader(ifs);
      FileStream ofs = new FileStream(mutatedSourceFile, FileMode.Create);
      StreamWriter sw = new StreamWriter(ofs);
      int currPos = 0;
      int currChar;
 
      while ((currChar = sr.Read()) != -1)
      {
        if (currPos == mutatePosition)
        {
          if ((char)currChar == '<') {
            sw.Write('>');
            Console.WriteLine("Mutating < to > at position " + currPos);
          }
          else if ((char)currChar == '>')
          {
            sw.Write('<');
            Console.WriteLine("Mutating > to < at position " + currPos);
          }
          else sw.Write((char)currChar);
        }
        else
          sw.Write((char)currChar);

        ++currPos;
       }
 
      sw.Close(); ofs.Close();
      sr.Close(); ifs.Close();
    } // CreateMutant

    static void BuildMutant(string mutantSolution, string devenv)
    {
      ProcessStartInfo psi =
          new ProcessStartInfo(devenv, mutantSolution + " /rebuild");
      Process p = new Process();

      p.StartInfo = psi; p.Start();
      while (p.HasExited == false) {
        System.Threading.Thread.Sleep(400);
        Console.WriteLine("Waiting for mutant build to complete . . ");
      }
      p.Close();
    } // BuildMutant()

    static void BuildTestProject(string testProject, string devenv)
    {
      ProcessStartInfo psi =
        new ProcessStartInfo(devenv, testProject + " /rebuild");
      Process p = new Process();

      p.StartInfo = psi;
      p.Start();
      while (p.HasExited == false)
      {
        System.Threading.Thread.Sleep(500);
        Console.WriteLine("Waiting for test project build to complete . . ");
      }
      p.Close();
    } // BuildTestProject()

    static void TestMutant(string testExecutable)
    {
      ProcessStartInfo psi = new ProcessStartInfo(testExecutable);
      Process p = new Process();
      p.StartInfo = psi;
      p.Start();
      while (p.HasExited == false)
      {
        System.Threading.Thread.Sleep(200);
        //Console.WriteLine("Running test cases . . ");
      }
      p.Close();
 
    } // TestMutant()

  } // Program
} // ns
